@@ -1,8 +1,3 @@
+from .abstract import AbstractWorker
 from .base import BaseWorker
 from .classic import ClassicWorker
-
-
-__all__ = [
-    "BaseWorker",
-    "ClassicWorker"
-]
