@@ -73,8 +73,12 @@ class RedisClientTaskCourierTestCase(RedisSetupMixin, unittest.TestCase):
                          self.task_courier.redis_connection)
 
     def test__generate_task_id(self):
+        self.assertFalse(hasattr(self.task_courier, "_uuid1_is_safe"))
         task_id = self.task_courier._generate_task_id()
-        self.assertEqual(type(task_id), uuid.UUID)
+        self.assertTrue(hasattr(self.task_courier, "_uuid1_is_safe"))
+        uuid1_is_safe = uuid.uuid1().is_safe is uuid.SafeUUID.safe
+        expected_version = 1 if uuid1_is_safe else 4
+        self.assertEqual(expected_version, task_id.version)
 
     def test_add_task_to_queue_as_dict(self):
         start_task_data = {
@@ -112,6 +116,66 @@ class RedisClientTaskCourierTestCase(RedisSetupMixin, unittest.TestCase):
         self.assertEqual(start_task_data, task_data)
         self.assertEqual(task_data("123"), "123123")
 
+    def test_add_task_to_queue_is_FIFO(self):
+        task = self.redis_connection.lpop(
+            self.task_courier._get_full_queue_name(
+                queue_name="test_queue_name", sufix="pending")
+        )
+        self.assertIsNone(task)  # queue is empty
+
+        start_task_ids = [
+            self.task_courier.add_task_to_queue(
+                queue_name="test_queue_name",
+                task_data=num) for num in range(100)
+        ]
+
+        for num in range(100):
+            task = self.redis_connection.lpop(
+                self.task_courier._get_full_queue_name(
+                    queue_name="test_queue_name", sufix="pending")
+            )
+            self.assertIsNotNone(task)
+            task_id, task_data = pickle.loads(task)
+
+            self.assertEqual(num, task_data)
+            self.assertEqual(start_task_ids[num], task_id)
+
+        task = self.redis_connection.lpop(
+            self.task_courier._get_full_queue_name(
+                queue_name="test_queue_name", sufix="pending")
+        )
+        self.assertIsNone(task)  # queue is empty
+
+    def test_bulk_add_tasks_to_queue_is_FIFO(self):
+        task = self.redis_connection.lpop(
+            self.task_courier._get_full_queue_name(
+                queue_name="test_queue_name", sufix="pending")
+        )
+        self.assertIsNone(task)  # queue is empty
+
+        start_tasks_data = [num for num in range(100)]
+        start_task_ids = self.task_courier.bulk_add_tasks_to_queue(
+            queue_name="test_queue_name",
+            tasks_data=start_tasks_data)
+
+        for num in range(100):
+            task = self.redis_connection.lpop(
+                self.task_courier._get_full_queue_name(
+                    queue_name="test_queue_name", sufix="pending")
+            )
+            self.assertIsNotNone(task)
+            task_id, task_data = pickle.loads(task)
+
+            self.assertEqual(num, task_data)
+            self.assertEqual(start_task_ids[num], task_id)
+            self.assertEqual(start_tasks_data[num], task_data)
+
+        task = self.redis_connection.lpop(
+            self.task_courier._get_full_queue_name(
+                queue_name="test_queue_name", sufix="pending")
+        )
+        self.assertIsNone(task)  # queue is empty
+
     def test_get_task_result_if_delete_data_True(self):
         before_task_result = "test_result_123"
         before_task_id = uuid.uuid1()
@@ -367,6 +431,55 @@ class RedisWorkerTaskCourierTestCase(RedisSetupMixin, unittest.TestCase):
         value = self.redis_connection.get(name)
         self.assertIsNone(value)
 
+    def test_bulk_return_task_results(self):
+        queue_name = "test_queue"
+
+        start_task_results = dict()
+        for num in range(100):
+            task_id = uuid.uuid1()
+            task_result = num
+            start_task_results[task_id] = task_result
+
+        self.task_courier.result_timeout = 100
+        self.task_courier.bulk_return_task_results(
+            queue_name=queue_name,
+            tasks=start_task_results
+        )
+
+        for task_id, task_result in start_task_results.items():
+            name = self.task_courier._get_full_queue_name(
+                queue_name=queue_name, sufix="results:") + str(task_id)
+            value = pickle.loads(self.redis_connection.get(name))
+            self.assertEqual(value, task_result)
+
+    def test_bulk_return_task_results_correct_timeout(self):
+        queue_name = "test_queue"
+
+        start_task_results = dict()
+        for num in range(100):
+            task_id = uuid.uuid1()
+            task_result = num
+            start_task_results[task_id] = task_result
+
+        self.task_courier.result_timeout = 2
+        self.task_courier.bulk_return_task_results(
+            queue_name=queue_name,
+            tasks=start_task_results
+        )
+
+        for task_id, task_result in start_task_results.items():
+            name = self.task_courier._get_full_queue_name(
+                queue_name=queue_name, sufix="results:") + str(task_id)
+            value = pickle.loads(self.redis_connection.get(name))
+            self.assertEqual(value, task_result)
+
+        time.sleep(2)
+        for task_id, task_result in start_task_results.items():
+            name = self.task_courier._get_full_queue_name(
+                queue_name=queue_name, sufix="results:") + str(task_id)
+            value = self.redis_connection.get(name)
+            self.assertIsNone(value, task_result)
+
     def test_wait_for_task(self):
         queue_name = "test_queue_name"
         before_task_id = uuid.uuid1()
@@ -470,16 +583,16 @@ class RedisWorkerTaskCourierTestCase(RedisSetupMixin, unittest.TestCase):
             self.assertEqual(before_task_id, after_task_id)
             self.assertEqual(before_task_data, after_task_data)
 
-    def test_get_tasks(self):
+    def test_bulk_get_tasks(self):
         queue_name = "test_queue_name"
         before_task_id = uuid.uuid1()
-        before_task_data = "test_get_tasks_data"
+        before_task_data = "test_bulk_get_tasks_data"
         task = pickle.dumps((before_task_id, before_task_data))
 
         self.redis_connection.rpush(
             self.task_courier._get_full_queue_name(
                 queue_name=queue_name, sufix="pending"), task)
-        tasks = self.task_courier.get_tasks(
+        tasks = self.task_courier.bulk_get_tasks(
             queue_name=queue_name, max_count=20)
         after_task_id, after_task_data = tasks[0]
 
@@ -488,20 +601,20 @@ class RedisWorkerTaskCourierTestCase(RedisSetupMixin, unittest.TestCase):
         self.assertEqual(before_task_id, after_task_id)
         self.assertEqual(before_task_data, after_task_data)
 
-    def test_get_tasks_if_no_tasks(self):
+    def test_bulk_get_tasks_if_no_tasks(self):
         queue_name = "test_queue_name"
-        tasks = self.task_courier.get_tasks(
+        tasks = self.task_courier.bulk_get_tasks(
             queue_name=queue_name, max_count=20)
         self.assertEqual(type(tasks), list)
         self.assertEqual(len(tasks), 0)
 
-    def test_get_tasks_if_many_tasks_and_FIFO(self):
+    def test_bulk_get_tasks_if_many_tasks_and_FIFO(self):
         queue_name = "test_queue_name"
         before_tasks = list()
 
         for num in range(50):
             task_id = uuid.uuid1()
-            task_data = f"test_get_tasks_data_{num}"
+            task_data = f"test_bulk_get_tasks_data_{num}"
             task = (task_id, task_data)
             before_tasks.append(task)
             task = pickle.dumps(task)
@@ -509,7 +622,7 @@ class RedisWorkerTaskCourierTestCase(RedisSetupMixin, unittest.TestCase):
                 self.task_courier._get_full_queue_name(
                     queue_name=queue_name, sufix="pending"), task)
 
-        after_tasks = self.task_courier.get_tasks(
+        after_tasks = self.task_courier.bulk_get_tasks(
             queue_name=queue_name, max_count=20)
 
         self.assertEqual(type(after_tasks), list)
